@@ -1,5 +1,5 @@
 /* Page fault handling library.
-   Copyright (C) 1998-1999, 2002, 2004-2011  Bruno Haible <bruno@clisp.org>
+   Copyright (C) 1998-1999, 2002, 2004-2011, 2016-2018  Bruno Haible <bruno@clisp.org>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,17 +18,34 @@
 #ifndef _SIGSEGV_H
 #define _SIGSEGV_H
 
+/* Get size_t.  */
+#include <stddef.h>
 
 
+
+/* Correct the value of SIGSTKSZ on some systems.
+   AIX 64-bit: original value 4096 is too small.
+   HP-UX: original value 8192 is too small.  */
+#if defined _AIX && defined _ARCH_PPC64
+# include <signal.h>
+# undef SIGSTKSZ
+# define SIGSTKSZ 8192
+#endif
+#if defined __hpux
+# include <signal.h>
+# undef SIGSTKSZ
+# define SIGSTKSZ 16384
+#endif
+
 /* HAVE_SIGSEGV_RECOVERY
    is defined if the system supports catching SIGSEGV.  */
-#if 1
+#if 0
 # define HAVE_SIGSEGV_RECOVERY 1
 #endif
 
 /* HAVE_STACK_OVERFLOW_RECOVERY
    is defined if stack overflow can be caught.  */
-#if 1
+#if 0
 # define HAVE_STACK_OVERFLOW_RECOVERY 1
 #endif
 
@@ -37,7 +54,7 @@
 extern "C" {
 #endif
 
-#define LIBSIGSEGV_VERSION 0x020A    /* version number: (major<<8) + minor */
+#define LIBSIGSEGV_VERSION 0x020C    /* version number: (major<<8) + minor */
 extern int libsigsegv_version;       /* Likewise */
 
 /* -------------------------------------------------------------------------- */
@@ -46,10 +63,21 @@ extern int libsigsegv_version;       /* Likewise */
  * The mask of bits that are set to zero in a fault address that gets passed
  * to a global SIGSEGV handler.
  * On some platforms, the precise fault address is not known, only the memory
- * page into which the fault address falls. On these platforms,
- * SIGSEGV_FAULT_ADDRESS_ALIGNMENT is greater than 1.
+ * page into which the fault address falls. This is apparently allowed by POSIX:
+ * <http://pubs.opengroup.org/onlinepubs/9699919799/basedefs/signal.h.html>
+ * says: "For some implementations, the value of si_addr may be inaccurate."
+ * In this case, the returned fault address is rounded down to a multiple of
+ * getpagesize() = sysconf(_SC_PAGESIZE).
+ * On such platforms, we define SIGSEGV_FAULT_ADDRESS_ALIGNMENT to be an upper
+ * bound for getpagesize() (and, like getpagesize(), also a power of 2).
+ * On the platforms where the returned fault address is the precise one, we
+ * define SIGSEGV_FAULT_ADDRESS_ALIGNMENT to 1.
  */
-#if defined __linux__ && (defined __s390__ || defined __s390x__)
+#if defined __NetBSD__ && (defined __sparc__ || defined __sparc64__)
+  /* getpagesize () is 0x1000 or 0x2000, depending on hardware.  */
+# define SIGSEGV_FAULT_ADDRESS_ALIGNMENT 0x2000UL
+#elif defined __linux__ && (defined __s390__ || defined __s390x__)
+  /* getpagesize () is 0x1000.  */
 # define SIGSEGV_FAULT_ADDRESS_ALIGNMENT 0x1000UL
 #else
 # define SIGSEGV_FAULT_ADDRESS_ALIGNMENT 1UL
@@ -115,7 +143,7 @@ extern void sigsegv_leave_handler (void);
  * on some platforms it is a 'struct sigcontext *', on others merely an
  * opaque 'void *'.
  */
-typedef struct sigcontext *stackoverflow_context_t;
+typedef void *stackoverflow_context_t;
 
 /*
  * The type of a stack overflow handler.
@@ -148,7 +176,7 @@ typedef void (*stackoverflow_handler_t) (int emergency, stackoverflow_context_t
  * overflow.
  */
 extern int stackoverflow_install_handler (stackoverflow_handler_t handler,
-                                          void* extra_stack, unsigned long extra_stack_size);
+                                          void* extra_stack, size_t extra_stack_size);
 
 /*
  * Deinstalls the stack overflow handler.
@@ -195,7 +223,7 @@ extern void sigsegv_init (sigsegv_dispatcher* dispatcher);
  * Returns a "ticket" that can be used to remove the handler later.
  */
 extern void* sigsegv_register (sigsegv_dispatcher* dispatcher,
-                               void* address, unsigned long len,
+                               void* address, size_t len,
                                sigsegv_area_handler_t handler, void* handler_arg);
 
 /*
